(* Euclide.mli *)
(* Denis Monasse 1998. *)

(*
Le fichier euclide.ml implemente et manipule en Caml Light les objets de
la geometrie euclidienne comme les points (definis par leur deux
coordonnees), les droites (definies par les coefficients u, v, et h d'une de
leurs equations ux+vy+h), les cercles (definis par les coodonnees de leur
centre et leur rayon), les triangles (definis comme des triplets de couples
de nombres reels) et, uniquement pour des raisons de dessin, les couleurs
et les epaisseurs de traits, avec la declaration de type

type point = {xc : float; yc : float};;
type droite = {u : float; v : float; h : float};;
type cercle = {xcentre : float;ycentre : float;rayon : float};;

type objet_geometrique =
     | Point of point | Droite of droite | Cercle of cercle
     | Triangle of (point*point*point) | Couleur of int | Epaisseur of int;;

Les fonctions suivantes permettent de manipuler ces objets:

###################################
Manipulations d'objets :

point : float -> float -> objet_geometrique
      point x y      retourne le point de coordonnees (x,y)
droite : float -> float -> float -> objet_geometrique
      droite u v h   retourne la droite d'equation ux+vy+h=0
cercle : float -> float -> float -> objet_geometrique
      cercle x y r   retourne le cercle de rayon r dont
le centre est le point (x,y)
random_point : float -> float -> float -> float
      random_point x1 x2 y1 y2   retourne un point au
hasard dans la fenetre delimitee
      par les abscisses x1 et x2  et les ordonnees y1 et y2
centre_de_cercle : objet_geometrique -> objet_geometrique
      centre_de_cercle c  retourne le point centre du
cercle c
rayon_de_cercle : objet_geometrique -> float
      rayon_de_cercle c   retourne le rayon du cercle c
cercle_par_centre_et_rayon : objet_geometrique -> float -> objet_geometrique
      cercle_par_centre_et_rayon m r  retourne le cercle ayant pour
centre le point m et de rayon r
point_objet : objet_geometrique -> float -> objet_geometrique
      point_objet obj t    retourne le point de parametre t de la
      droite ou du cercle obj
triangle : objet_geometrique -> objet_geometrique -> objet_geometrique ->
objet_geometrique
      triangle m1 m2 m3  retourne le triangle ayant pour sommets les
      points m1, m2, m3 
      triangle d1 d2 d3  retourne le triangle dont les cotes sont
      portes par les droites d1, d2 et d3
sommets : objet_geometrique -> objet_geometrique vect
      sommets triang   retourne un vecteur dont les trois composantes
      sont les points sommets du triangle

###################################
Operations sur les objets :

droite_par_2_points : objet_geometrique -> objet_geometrique ->
objet_geometrique
      droite_par_2_points m1 m2  (ou m1 *? m2) retourne
      la droite passant par les points m1 et m2
cercle_par_centre_et_point : objet_geometrique -> objet_geometrique ->
objet_geometrique
      cercle_par_centre_et_point m1 m2  retourne le cercle ayant pour
 centre le point m1 
      et passant par le point m2
axe_radical : objet_geometrique -> objet_geometrique -> objet_geometrique
      axe_radical c1 c2  (ou c1 +? c2) retourne la droite
qui est l'axe radical des deux
      cercles c1 et c2
droite_parallele : objet_geometrique -> objet_geometrique -> objet_geometriq=
ue
      droite_parallele m d   (ou m //? d) retourne la
droite parallele a d passant par
      le point m; les deux parametres peuvent etre
echanges sans inconvenient
droite_perpendiculaire : objet_geometrique -> objet_geometrique ->
objet_geometrique
      droite_perpendiculaire m d  (ou m +? d) retourne la
droite perpendiculaire a d
      passant par le point m; les deux parametres peuvent
etre echanges sans inconvenient
milieu : objet_geometrique -> objet_geometrique -> objet_geometrique
      milieu m1 m2   retourne le point milieu des deux
points m1 et m2
mediatrice : objet_geometrique -> objet_geometrique -> objet_geometrique
      mediatrice m1 m2  (ou m1 +? m2) retourne la droite
mediatrice des deux points m1 et
      m2
bissectrices : objet_geometrique -> objet_geometrique -> objet_geometrique v=
ect
      bissectrices d1 d2  (ou d1 **? d2) retourne un
vecteur [|d'1 ; d'2|] contenant les
      deux droites bissectrices des deux droites d1 et d2
projection : objet_geometrique -> objet_geometrique -> objet_geometrique
      projection m obj  (ou m *? obj)  retourne le point
projection du point m sur
      l'objet obj qui peut-etre une droite ou un cercle;
; les deux parametres peuvent
      etre echanges sans inconvenient
point_d_intersection : objet_geometrique -> objet_geometrique ->
objet_geometrique
      point_d_intersection d1 d2  (ou d1 *? d2)  retourne
le point d'intersection des
      deux droites d1 et d2
points_d_intersection : objet_geometrique -> objet_geometrique ->
objet_geometrique vect
      points_d_intersection obj1 obj2  (ou  obj1 **?
obj2)  retourne un vecteur dont les
      deux composantes sont les points d'intersection des
objets obj1 et obj2,
      l'un etant uncercle, l'autre etant soit un cercle
soit une droite
symetrique : objet_geometrique -> objet_geometrique -> objet_geometrique
      symetrique obj1 obj2  (ou  obj1 /? obj2) renvoie
l'objet  symetrique de obj1
       (point, cercle ou droite) par rapport a obj2
(point, cercle ou droite);
       la symetrie par rapport a un cercle est
l'inversion qui laisse stable ce cercle

###################################
Operations sur les triangles:

centre_de_gravite : objet_geometrique -> objet_geometrique
centre_du_cercle_circonscrit : objet_geometrique -> objet_geometrique
cercle_circonscrit : objet_geometrique -> objet_geometrique
centre_du_cercle_inscrit : objet_geometrique -> objet_geometrique
cercle_inscrit : objet_geometrique -> objet_geometrique
orthocentre : objet_geometrique -> objet_geometrique
cercle_des_9_points : objet_geometrique -> objet_geometrique
droite_Euler : objet_geometrique -> objet_geometrique
      ces fonctions recoivent un triangle comme parametre
et renvoient l'objet
      geometrique correspondant
centres_des_cercles_exinscrits : objet_geometrique -> objet_geometrique vect
cercles_exinscrits : objet_geometrique -> objet_geometrique vect
      ces fonctions recoivent un triangle comme parametre
et renvoient un vecteur
      dont les trois composantes sont les objets
geometriques correspondants

###################################
Quelques tests geometriques:

appartient : objet_geometrique -> objet_geometrique -> bool
      appartient m obj   teste si un point m appartient
(approximativement) a
      l'objet obj (une droite ou un cercle)
alignes : objet_geometrique -> objet_geometrique -> objet_geometrique -> boo=
l
      alignes m1 m2 m3   teste si les trois points m1,m2
et m3 sont (approximativement)
      alignes
concourantes : objet_geometrique -> objet_geometrique -> objet_geometrique
-> bool
      concourantes d1 d2 d3   teste si les trois droites
d1, d2 et d3 sont
      (approximativement) concourantes
appartient_tol : float -> objet_geometrique -> objet_geometrique -> bool
      appartient_tol eps m obj   teste si un point m
appartient (approximativement) a
      l'objet obj (une droite ou un cercle) avec la
tolerance  eps
alignes_tol : float -> objet_geometrique -> objet_geometrique ->
objet_geometrique -> bool
      alignes_tol eps  m1 m2 m3   teste si les trois
points m1,m2 et m3 sont
      (approximativement) alignes avec la tolerance  eps
concourantes_tol : float -> objet_geometrique -> objet_geometrique ->
objet_geometrique -> bool
      concourantes_tol eps d1 d2 d3   teste si les trois
droites d1, d2 et d3 sont
      (approximativement) concourantes avec la tolerance  eps

###################################
Quelques fonctions graphiques:

fenetre : string -> float -> float -> float -> float
      fenetre param x1 x2 y1 y2    ouvre une fenetre
graphique en passant  param a la
      fonction  open_graph  et definit la fenetre
mathematique par les abscisses x1,x2 et
      les ordonnees y1,y2. A vous d'ajuster ces
parametres pour que le repere soit
      orthonorme.
dessine_objet : objet_geometrique -> unit
dessine : objet_geometrique list -> unit
      dessine un objet ou une liste d'objets

Un exemple est donn dans le fichier exemple.ml

###################################
Auteur: Denis Monasse
monasse@cicrp.jussieu.fr

*)

type point = {xc : float; yc : float};;
type droite = {u : float; v : float; h : float};;
type cercle = {xcentre : float; ycentre : float; rayon : float};;

type objet_geometrique =
     | Point of point | Droite of droite | Cercle of cercle
     | Triangle of (point*point*point) | Couleur of int | Epaisseur of int;;

exception Erreur of string;;

value 
point : float -> float -> objet_geometrique and
droite : float -> float -> float -> objet_geometrique and
cercle : float -> float -> float -> objet_geometrique and
axe_Ox : objet_geometrique and
axe_Oy : objet_geometrique and
random_point : float -> float -> float -> float -> objet_geometrique and
droite_par_2_points :
 objet_geometrique -> objet_geometrique -> objet_geometrique and
point_objet : objet_geometrique -> float -> objet_geometrique and
centre_de_cercle : objet_geometrique -> objet_geometrique and
rayon_de_cercle : objet_geometrique -> float and
cercle_par_centre_et_rayon :
 objet_geometrique -> float -> objet_geometrique and
cercle_par_centre_et_point :
 objet_geometrique -> objet_geometrique -> objet_geometrique and
axe_radical : objet_geometrique -> objet_geometrique -> objet_geometrique and
point_d_intersection :
 objet_geometrique -> objet_geometrique -> objet_geometrique and
points_d_intersection :
 objet_geometrique -> objet_geometrique -> objet_geometrique vect and
triangle :
 objet_geometrique ->
 objet_geometrique -> objet_geometrique -> objet_geometrique and
sommets : objet_geometrique -> objet_geometrique vect and
droite_parallele :
 objet_geometrique -> objet_geometrique -> objet_geometrique and
droite_perpendiculaire :
 objet_geometrique -> objet_geometrique -> objet_geometrique and
milieu : objet_geometrique -> objet_geometrique -> objet_geometrique and
mediatrice : objet_geometrique -> objet_geometrique -> objet_geometrique and
bissectrices :
 objet_geometrique -> objet_geometrique -> objet_geometrique vect and
projection : objet_geometrique -> objet_geometrique -> objet_geometrique and
symetrique : objet_geometrique -> objet_geometrique -> objet_geometrique and
centre_de_gravite : objet_geometrique -> objet_geometrique and
centre_du_cercle_circonscrit : objet_geometrique -> objet_geometrique and
cercle_circonscrit : objet_geometrique -> objet_geometrique and
centre_du_cercle_inscrit : objet_geometrique -> objet_geometrique and
cercle_inscrit : objet_geometrique -> objet_geometrique and
orthocentre : objet_geometrique -> objet_geometrique and
cercle_des_9_points : objet_geometrique -> objet_geometrique and
droite_Euler : objet_geometrique -> objet_geometrique and
centres_des_cercles_exinscrits :
 objet_geometrique -> objet_geometrique vect and
cercles_exinscrits : objet_geometrique -> objet_geometrique vect and
appartient_tol : float -> objet_geometrique -> objet_geometrique -> bool and
appartient : objet_geometrique -> objet_geometrique -> bool and
alignes_tol :
 float -> objet_geometrique -> objet_geometrique -> objet_geometrique ->
bool and
alignes :
 objet_geometrique -> objet_geometrique -> objet_geometrique -> bool and
concourantes_tol :
 float -> objet_geometrique -> objet_geometrique -> objet_geometrique ->
bool and
concourantes :
 objet_geometrique -> objet_geometrique -> objet_geometrique -> bool and
fenetre : string -> float -> float -> float -> float -> unit and
dessine_objet : objet_geometrique -> unit and
dessine : objet_geometrique list -> unit and
prefix *? : objet_geometrique -> objet_geometrique -> objet_geometrique and
prefix //? : objet_geometrique -> objet_geometrique -> objet_geometrique and
prefix **? : objet_geometrique -> objet_geometrique -> objet_geometrique
vect and
prefix +? : objet_geometrique -> objet_geometrique -> objet_geometrique and
prefix /? : objet_geometrique -> objet_geometrique -> objet_geometrique;;
